# Copyright 2017 Mycroft AI Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import inspect
import logging
import sys


def _make_log_method(fn):
    @classmethod
    def method(cls, *args, **kwargs):
        cls._log(fn, *args, **kwargs)

    method.__func__.__doc__ = fn.__doc__
    return method


class LOG:
    """
    Custom logger class that acts like logging.Logger
    The logger name is automatically generated by the module of the caller

    Usage:
        >>> LOG.debug('My message: %s', debug_str)
        13:12:43.673 - :<module>:1 - DEBUG - My message: hi
        >>> LOG('custom_name').debug('Another message')
        13:13:10.462 - custom_name - DEBUG - Another message
    """

    _custom_name = None
    handler = None
    level = None

    # Copy actual logging methods from logging.Logger
    # Usage: LOG.debug(message)
    debug = _make_log_method(logging.Logger.debug)
    info = _make_log_method(logging.Logger.info)
    warning = _make_log_method(logging.Logger.warning)
    error = _make_log_method(logging.Logger.error)
    exception = _make_log_method(logging.Logger.exception)

    @classmethod
    def init(cls):
        cls.level = "DEBUG"
        fmt = '%(asctime)s.%(msecs)03d - ' \
              '%(name)s - %(levelname)s - %(message)s'
        datefmt = '%H:%M:%S'
        formatter = logging.Formatter(fmt, datefmt)
        cls.handler = logging.StreamHandler(sys.stdout)
        cls.handler.setFormatter(formatter)

        # Enable logging in external modules
        cls.create_logger('').setLevel(cls.level)

    @classmethod
    def create_logger(cls, name):
        logger = logging.getLogger(name)
        logger.propagate = False
        logger.addHandler(cls.handler)
        return logger

    def __init__(self, name):
        LOG._custom_name = name

    @classmethod
    def _log(cls, func, *args, **kwargs):
        if cls._custom_name is not None:
            name = cls._custom_name
            cls._custom_name = None
        else:
            # Stack:
            # [0] - _log()
            # [1] - debug(), info(), warning(), or error()
            # [2] - caller
            stack = inspect.stack()

            # Record:
            # [0] - frame object
            # [1] - filename
            # [2] - line number
            # [3] - function
            # ...
            record = stack[2]
            mod = inspect.getmodule(record[0])
            module_name = mod.__name__ if mod else ''
            name = module_name + ':' + record[3] + ':' + str(record[2])
        func(cls.create_logger(name), *args, **kwargs)


LOG.init()



from json_database import JsonDatabase
from os.path import expanduser, join, isdir
from os import makedirs


class ContactExists(ValueError):
    """ Choose new contact name """


class ContactDoesNotExist(ValueError):
    """ Choose new contact name """


class ContactList:
    def __init__(self, database_name="contacts.db"):
        db_dir = join(expanduser("~"), ".baresip")
        if not isdir(db_dir):
            makedirs(db_dir)
        self.db_path = join(db_dir, database_name)

    def import_baresip_contacts(self):
        db_dir = join(expanduser("~"), ".baresip", "contacts")
        with open(db_dir) as f:
            lines = f.readlines()

        for line in lines:
            if line.startswith("#"):
                continue
            user, address = line.split("<")
            user = user.replace('"', "")
            address = address.split(">")[0]
            with JsonDatabase("contacts", self.db_path) as db:
                users = db.search_by_value("url", address)
            if not users:
                self.add_contact(user, address)
            else:
                self.update_contact(user, address)

    def export_baresip_contacts(self):
        db_dir = join(expanduser("~"), ".baresip", "contacts")
        with JsonDatabase("contacts", self.db_path) as db:
            users = db.search_by_key("url")
        with open(db_dir) as f:
            lines = f.readlines()
        for user in users:
            line = "\"{name}\" <{address}>".format(name=user["name"],
                                               address=user["url"])
            if line not in lines:
                lines.append(line + "\n")
        with open(db_dir, "w") as f:
            f.writelines(lines)

    def search_contact(self, url):
        with JsonDatabase("contacts", self.db_path) as db:
            users = db.search_by_value("url", url)
            if len(users):
                return users[0]
        return None

    def is_contact(self, url):
        return self.search_contact(url) is not None

    def get_contact(self, name):
        with JsonDatabase("contacts", self.db_path) as db:
            users = db.search_by_value("name", name)
            if len(users):
                return users[0]
        return None

    def add_contact(self, name, url):
        if self.get_contact(name) is not None:
            raise ContactExists
        with JsonDatabase("contacts", self.db_path) as db:
            user = {"name": name, "url": url}
            db.add_item(user)

    def update_contact(self, name, url):
        contact = self.get_contact(name)
        if contact is None:
            raise ContactDoesNotExist

        with JsonDatabase("contacts", self.db_path) as db:
            item_id = db.get_item_id(contact)
            db.update_item(item_id, {"name": name, "url": url})

    def remove_contact(self, name):
        contact = self.get_contact(name)
        if contact is None:
            raise ContactDoesNotExist
        with JsonDatabase("contacts", self.db_path) as db:
            item_id = db.get_item_id(contact)
            db.remove_item(item_id)

    def print_contacts(self):
        with JsonDatabase("contacts", self.db_path) as db:
            db.print()

    def list_contacts(self):
        with JsonDatabase("contacts", self.db_path) as db:
            users = db.search_by_key("url")
        return users



from setuptools import setup

setup(
    name='baresipy',
    version='0.1.2',
    packages=['baresipy', 'baresipy.utils'],
    url='https://github.com/JarbasAl/baresipy',
    install_requires=["ResponsiveVoice",
                      "opentone>=0.1.2",
                      "pydub",
                      "pexpect"],
    license='MIT',
    author='jarbasAI',
    author_email='jarbasai@mailfence.com',
    description='python baresip wrapper'
)
